@@ -2,8 +2,10 @@ import logging
 import os
 from argparse import ArgumentParser
 
-from erdpy import config, dependencies, errors, flows, nodedebug, projects, ide, proxy
+from erdpy import (config, dependencies, errors, facade, ide, nodedebug,
+                   projects, proxy, transactions)
 from erdpy._version import __version__
+import sys
 
 logger = logging.getLogger("cli")
 
@@ -15,7 +17,7 @@ def main():
     if args.verbose:
         logging.basicConfig(level=logging.DEBUG)
     else:
-        logging.basicConfig(level=logging.WARNING)
+        logging.basicConfig(level=logging.WARN)
 
     if not hasattr(args, "func"):
         parser.print_help()
@@ -31,8 +33,7 @@ def setup_parser():
     parser.add_argument("--verbose", action="store_true", default=False)
 
     install_parser = subparsers.add_parser("install")
-    choices = ["C_BUILDCHAIN", "SOL_BUILDCHAIN",
-               "RUST_BUILDCHAIN", "NODE_DEBUG"]
+    choices = ["C_BUILDCHAIN", "SOL_BUILDCHAIN", "RUST_BUILDCHAIN", "ARWENTOOLS"]
     install_parser.add_argument("group", choices=choices)
     install_parser.set_defaults(func=install)
 
@@ -56,24 +57,37 @@ def setup_parser():
     # TODO: path to project or path to bytecode (hex.arwen).
     deploy_parser.add_argument("project", nargs='?', default=os.getcwd())
     deploy_parser.add_argument("--proxy", required=True)
-    deploy_parser.add_argument("--owner", required=True)
     deploy_parser.add_argument("--pem", required=True)
     deploy_parser.add_argument("--arguments", nargs='+')
     deploy_parser.add_argument("--gas-price", default=config.DEFAULT_GASPRICE)
     deploy_parser.add_argument("--gas-limit", default=config.DEFAULT_GASLIMIT)
+    deploy_parser.add_argument("--value", default="0")
+    deploy_parser.add_argument("--metadata-upgradeable", action="store_true", default=False)
     deploy_parser.set_defaults(func=deploy)
 
     call_parser = subparsers.add_parser("call")
     call_parser.add_argument("contract")
     call_parser.add_argument("--proxy", required=True)
-    call_parser.add_argument("--caller", required=True)
     call_parser.add_argument("--pem", required=True)
     call_parser.add_argument("--function", required=True)
     call_parser.add_argument("--arguments", nargs='+')
     call_parser.add_argument("--gas-price", default=config.DEFAULT_GASPRICE)
     call_parser.add_argument("--gas-limit", default=config.DEFAULT_GASLIMIT)
+    call_parser.add_argument("--value", default="0")
     call_parser.set_defaults(func=call)
 
+    upgrade_parser = subparsers.add_parser("upgrade")
+    upgrade_parser.add_argument("contract")
+    upgrade_parser.add_argument("project")
+    upgrade_parser.add_argument("--proxy", required=True)
+    upgrade_parser.add_argument("--pem", required=True)
+    upgrade_parser.add_argument("--arguments", nargs='+')
+    upgrade_parser.add_argument("--gas-price", default=config.DEFAULT_GASPRICE)
+    upgrade_parser.add_argument("--gas-limit", default=config.DEFAULT_GASLIMIT)
+    upgrade_parser.add_argument("--value", default="0")
+    upgrade_parser.add_argument("--metadata-upgradeable", action="store_true", default=False)
+    upgrade_parser.set_defaults(func=upgrade)
+
     query_parser = subparsers.add_parser("query")
     query_parser.add_argument("contract")
     query_parser.add_argument("--proxy", required=True)
@@ -131,25 +145,51 @@ def setup_parser():
     ide_parser.add_argument("workspace", nargs='?', default=os.getcwd())
     ide_parser.set_defaults(func=run_ide)
 
+    tx_prepare_parser = subparsers.add_parser("tx-prepare")
+    tx_prepare_parser.add_argument("--pem", required=True)
+    tx_prepare_parser.add_argument("--nonce", required=True)
+    tx_prepare_parser.add_argument("--value", default="0")
+    tx_prepare_parser.add_argument("--receiver", required=True)
+    tx_prepare_parser.add_argument("--gas-price", default=config.DEFAULT_GASPRICE)
+    tx_prepare_parser.add_argument("--gas-limit", default=config.DEFAULT_GASLIMIT)
+    tx_prepare_parser.add_argument("--data", default="")
+    tx_prepare_parser.add_argument("--tag", default="untitled")
+    tx_prepare_parser.add_argument("workspace", nargs='?', default=os.getcwd())
+    tx_prepare_parser.set_defaults(func=tx_prepare)
+
+    tx_send_parser = subparsers.add_parser("tx-send")
+    tx_send_parser.add_argument("tx")
+    tx_send_parser.add_argument("--proxy", required=True)
+    tx_send_parser.set_defaults(func=tx_send)
+
+    tx_prepare_and_send_parser = subparsers.add_parser("tx-prepare-and-send")
+    tx_prepare_and_send_parser.add_argument("--pem", required=True)
+    tx_prepare_and_send_parser.add_argument("--value", default="0")
+    tx_prepare_and_send_parser.add_argument("--receiver", required=True)
+    tx_prepare_and_send_parser.add_argument("--gas-price", default=config.DEFAULT_GASPRICE)
+    tx_prepare_and_send_parser.add_argument("--gas-limit", default=config.DEFAULT_GASLIMIT)
+    tx_prepare_and_send_parser.add_argument("--data", default="")
+    tx_prepare_and_send_parser.add_argument("--proxy", required=True)
+    tx_prepare_and_send_parser.set_defaults(func=tx_prepare_and_send)
+
+    bech32_parser = subparsers.add_parser("bech32")
+    bech32_parser.add_argument("value")
+    group = bech32_parser.add_mutually_exclusive_group(required=True)
+    group.add_argument("--encode", action="store_true")
+    group.add_argument("--decode", action="store_true")
+    bech32_parser.set_defaults(func=do_bech32)
+
     return parser
 
 
 def install(args):
     group = args.group
-
-    try:
-        dependencies.install_group(group, overwrite=True)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    dependencies.install_group(group, overwrite=True)
 
 
 def list_templates(args):
     json = args.json
-
-    try:
-        projects.list_project_templates(json)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    projects.list_project_templates(json)
 
 
 def create(args):
@@ -157,10 +197,7 @@ def create(args):
     template = args.template
     directory = args.directory
 
-    try:
-        projects.create_from_template(name, template, directory)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    projects.create_from_template(name, template, directory)
 
 
 def build(args):
@@ -171,147 +208,105 @@ def build(args):
         "verbose": args.verbose
     }
 
-    try:
-        projects.build_project(project, options)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    projects.build_project(project, options)
 
 
 def deploy(args):
-    project = args.project
-    owner = args.owner
-    pem = args.pem
-    proxy_url = args.proxy
-    arguments = args.arguments
-    gas_price = args.gas_price
-    gas_limit = args.gas_limit
-
-    try:
-        flows.deploy_smart_contract(project, owner, pem, proxy_url, arguments, gas_price, gas_limit)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.deploy_smart_contract(args)
 
 
 def call(args):
-    contract = args.contract
-    caller = args.caller
-    pem = args.pem
-    proxy_url = args.proxy
-    function = args.function
-    arguments = args.arguments
-    gas_price = args.gas_price
-    gas_limit = args.gas_limit
+    facade.call_smart_contract(args)
 
-    try:
-        flows.call_smart_contract(contract, caller, pem, proxy_url, function, arguments, gas_price, gas_limit)
-    except errors.KnownError as err:
-        logger.fatal(err)
+
+def upgrade(args):
+    facade.upgrade_smart_contract(args)
 
 
 def query(args):
-    contract = args.contract
-    proxy_url = args.proxy
-    function = args.function
-    arguments = args.arguments
-
-    try:
-        flows.query_smart_contract(contract, proxy_url, function, arguments)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.query_smart_contract(args)
 
 
 def get_account(args):
     proxy_url = args.proxy
     address = args.address
-    try:
-        if args.balance:
-            flows.get_account_balance(proxy_url, address)
-        elif args.nonce:
-            flows.get_account_nonce(proxy_url, address)
-        else:
-            flows.get_account(proxy_url, address)
-    except errors.KnownError as err:
-        logger.fatal(err)
+
+    if args.balance:
+        facade.get_account_balance(proxy_url, address)
+    elif args.nonce:
+        facade.get_account_nonce(proxy_url, address)
+    else:
+        facade.get_account(proxy_url, address)
 
 
 def get_transaction_cost(args):
-    try:
-        flows.get_transaction_cost(args)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.get_transaction_cost(args)
 
 
 def get_num_shards(args):
     proxy_url = args.proxy
-
-    try:
-        flows.get_num_shards(proxy_url)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.get_num_shards(proxy_url)
 
 
 def get_last_block_nonce(args):
     proxy_url = args.proxy
     shard_id = args.shard_id
-
-    try:
-        flows.get_last_block_nonce(proxy_url, shard_id)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.get_last_block_nonce(proxy_url, shard_id)
 
 
 def get_gas_price(args):
     proxy_url = args.proxy
-
-    try:
-        flows.get_gas_price(proxy_url)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.get_gas_price(proxy_url)
 
 
 def get_chain_id(args):
     proxy_url = args.proxy
-
-    try:
-        flows.get_chain_id(proxy_url)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    facade.get_chain_id(proxy_url)
 
 
 def do_nodedebug(args):
     stop = args.stop
     restart = args.restart
 
-    try:
-        if restart:
-            nodedebug.stop()
-            nodedebug.start()
-        elif stop:
-            nodedebug.stop()
-        else:
-            nodedebug.start()
-    except errors.KnownError as err:
-        logger.fatal(err)
+    if restart:
+        nodedebug.stop()
+        nodedebug.start()
+    elif stop:
+        nodedebug.stop()
+    else:
+        nodedebug.start()
 
 
 def run_tests(args):
     project = args.project
     wildcard = args.wildcard
-
-    try:
-        projects.run_tests(project, wildcard)
-    except errors.KnownError as err:
-        logger.fatal(err)
+    projects.run_tests(project, wildcard)
 
 
 def run_ide(args):
     workspace = args.workspace
+    ide.run_ide(workspace)
 
-    try:
-        ide.run_ide(workspace)
-    except errors.KnownError as err:
-        logger.fatal(err)
+
+def tx_prepare(args):
+    transactions.prepare(args)
+
+
+def tx_send(args):
+    facade.send_prepared_transaction(args)
+
+
+def tx_prepare_and_send(args):
+    facade.prepare_and_send_transaction(args)
+
+
+def do_bech32(args):
+    facade.do_bech32(args)
 
 
 if __name__ == "__main__":
-    main()
+    try:
+        main()
+    except errors.KnownError as err:
+        logger.fatal(err)
+        sys.exit(1)
