@@ -5,11 +5,11 @@ import shutil
 from os import path
 from pathlib import Path
 
-from texttable import Texttable
-
-from erdpy import dependencies, workstation, errors, utils
+from erdpy import dependencies, errors, utils
 from erdpy.projects import shared
+from erdpy.projects.project_rust import CargoFile
 from erdpy.projects.templates_config import get_templates_repositories
+from texttable import Texttable
 
 logger = logging.getLogger("projects.templates")
 
@@ -43,6 +43,8 @@ class TemplateSummary():
 
 
 def create_from_template(name, template_name, directory):
+    directory = path.expanduser(directory)
+
     logger.info("create_from_template.name: %s", name)
     logger.info("create_from_template.template_name: %s", template_name)
     logger.info("create_from_template.directory: %s", directory)
@@ -55,8 +57,9 @@ def create_from_template(name, template_name, directory):
 
     _download_templates_repositories()
     _copy_template(template_name, project_directory)
+
     template = _load_as_template(project_directory)
-    template.apply(name)
+    template.apply(template_name, name)
 
     logger.info("Project created, template applied.")
 
@@ -88,7 +91,8 @@ class Template:
     def __init__(self, directory):
         self.directory = directory
 
-    def apply(self, project_name):
+    def apply(self, template_name, project_name):
+        self.template_name = template_name
         self.project_name = project_name
         self._extend()
         self._replace_placeholders()
@@ -125,19 +129,70 @@ class TemplateRust(Template):
         self.rust_bin_directory = path.join(self.rust_directory, "bin")
 
         cargo_path = path.join(self.directory, "Cargo.toml")
+        cargo_debug_path = path.join(self.directory, "debug", "Cargo.toml")
         launch_path = path.join(self.directory, ".vscode", "launch.json")
         tasks_path = path.join(self.directory, ".vscode", "tasks.json")
-        self._replace_placeholders_in_file(cargo_path)
-        self._replace_placeholders_in_file(launch_path)
-        self._replace_placeholders_in_file(tasks_path)
+        debug_main_path = path.join(self.directory, "debug", "src", "main.rs")
 
-    def _replace_placeholders_in_file(self, filepath):
-        content = utils.read_file(filepath)
-        content = content.replace("{{PROJECT_NAME}}", self.project_name)
-        content = content.replace("{{PATH_RUST_BIN}}", self.rust_bin_directory)
-        content = content.replace("{{RUSTUP_HOME}}", self.rust_directory)
-        content = content.replace("{{CARGO_HOME}}", self.rust_directory)
-        utils.write_file(filepath, content)
+        logger.info("Updating cargo files...")
+
+        cargo_file = CargoFile(cargo_path)
+        cargo_file.package_name = self.project_name
+        cargo_file.bin_name = self.project_name
+        cargo_file.version = "0.0.1"
+        cargo_file.authors = ["you"]
+        cargo_file.edition = "2018"
+        cargo_file.save()
+
+        cargo_file_debug = CargoFile(cargo_debug_path)
+        cargo_file_debug.package_name = f"{self.project_name}-debug"
+        cargo_file_debug.version = "0.0.1"
+        cargo_file_debug.authors = ["you"]
+        cargo_file_debug.edition = "2018"
+        cargo_file_debug.save()
+
+        logger.info("Applying replacements...")
+
+        self._replace_in_files(
+            [launch_path, tasks_path],
+            [
+                ("{{PROJECT_NAME}}", self.project_name),
+                ("{{PATH_RUST_BIN}}", self.rust_bin_directory),
+                ("{{RUSTUP_HOME}}", self.rust_directory),
+                ("{{CARGO_HOME}}", self.rust_directory)
+            ])
+
+        self._replace_in_files(
+            [debug_main_path],
+            [
+                # Example "use simple_coin::*" to "use my_project::*"
+                (f"use {self.template_name.replace('-', '_')}::*", f"use {self.project_name.replace('-', '_')}::*")
+            ]
+        )
+
+        self._replace_in_files(
+            [cargo_debug_path],
+            [
+                (f"[dependencies.{self.template_name}]", f"[dependencies.{self.project_name}]")
+            ]
+        )
+
+        json_test_files = list(Path(self.directory, "test").rglob("*.json"))
+        self._replace_in_files(
+            json_test_files,
+            [
+                (f"file:{self.template_name}.wasm", f"file:../target/wasm32-unknown-unknown/release/{self.project_name}.wasm")
+            ]
+        )
+
+    def _replace_in_files(self, files, replacements):
+        for file in files:
+            content = utils.read_file(file)
+
+            for to_replace, replacement in replacements:
+                content = content.replace(to_replace, replacement)
+
+            utils.write_file(file, content)
 
 
 class TemplateSol(Template):
